"use strict";

import React, { memo, useMemo, useState } from 'react';
import { StyleSheet, View } from 'react-native';
import { getOuterShadowOffset, getBackgroundColor, getShadowProperty, isLinearProps, numerify } from "../utils.js";
import { COMMON_STYLES } from "../constants.js";
import { Canvas, Shadow } from '@shopify/react-native-skia';
import LinearGradientFill from "./ShadowLinearGradientFill.js";
import { CornerRadii } from "./CornerRadii.js";

/**
 * A unified interface for both "solid" (InnerShadow) and "linear" (LinearShadow).
 * We automatically detect "linear mode" by checking if the user provides
 * gradient props (colors, from, to, etc.).
 */
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const UnifiedShadowView = /*#__PURE__*/memo(function UnifiedShadowView({
  width: propWidth,
  height: propHeight,
  inset,
  isReflectedLightEnabled,
  style,
  backgroundColor,
  shadowOffset,
  shadowColor,
  shadowBlur,
  reflectedLightOffset,
  reflectedLightColor,
  reflectedLightBlur,
  onLayout: propsOnLayout,
  children,
  ...props
}) {
  // Extract base fields
  const flatStyle = useMemo(() => StyleSheet.flatten(style) || {}, [style]);
  const bgColor = getBackgroundColor({
    backgroundColor,
    styleBackground: flatStyle.backgroundColor
  });
  const shadowProps = getShadowProperty({
    shadowOffset,
    shadowColor,
    shadowBlur,
    reflectedLightOffset,
    reflectedLightColor,
    reflectedLightBlur
  });
  const isLinear = isLinearProps(props);

  // If isReflectedLightEnabled is undefined, default to `props.inset` (typical).
  const _isReflectedLightEnabled = isReflectedLightEnabled ?? inset;
  const initialW = propWidth ?? numerify(flatStyle.width, 0);
  const initialH = propHeight ?? numerify(flatStyle.height, 0);
  const [layout, setLayout] = useState({
    width: initialW,
    height: initialH
  });
  const onLayout = React.useCallback(e => {
    propsOnLayout?.(e);
    if (initialW && initialH) return;
    const {
      width,
      height
    } = e.nativeEvent.layout;
    setLayout(prev => prev.width === width && prev.height === height ? prev : {
      width,
      height
    });
  }, [initialW, initialH, propsOnLayout]);
  // Create offset style for outer shadow if needed
  const outerShadowOffset = React.useMemo(() => getOuterShadowOffset({
    ...shadowProps,
    inset
  }), [shadowProps, inset]);

  // 현재 적용할 실제 크기 (명시 prop 우선, 아니면 측정값)
  const finalWidth = layout.width;
  const finalHeight = layout.height;
  const canRenderCanvas = finalWidth && finalHeight;
  return /*#__PURE__*/_jsxs(View, {
    ...props,
    style: [outerShadowOffset,
    // Possibly sets outer drop-shadow if needed
    COMMON_STYLES.canvasWrapper, style],
    onLayout: onLayout,
    children: [canRenderCanvas ? /*#__PURE__*/_jsx(Canvas, {
      style: [COMMON_STYLES.canvas, {
        width: finalWidth,
        height: finalHeight
      }],
      children: /*#__PURE__*/_jsxs(CornerRadii, {
        width: finalWidth,
        height: finalHeight,
        style: style,
        backgroundColor: bgColor,
        children: [isLinear && /*#__PURE__*/_jsx(LinearGradientFill, {
          ...props,
          // from, to, colors, etc.
          width: finalWidth,
          height: finalHeight
        }), inset && /*#__PURE__*/_jsx(Shadow, {
          dx: shadowProps.shadowOffset.width,
          dy: shadowProps.shadowOffset.height,
          blur: shadowProps.shadowBlur,
          color: shadowProps.shadowColor,
          inner: true
        }), _isReflectedLightEnabled && /*#__PURE__*/_jsx(Shadow, {
          dx: shadowProps.reflectedLightOffset.width,
          dy: shadowProps.reflectedLightOffset.height,
          blur: shadowProps.reflectedLightBlur,
          color: shadowProps.reflectedLightColor,
          inner: true
        })]
      })
    }) : null, children]
  });
});

/**
 * ShadowView: for a basic “solid” background shadow(no gradient props).
 *
 * @remarks
 * See {@link InnerShadowProps} for a linear gradient background shadow.
 *
 * @example
 * ```ts
 * <ShadowView style={styles.shadowView} inset>
 *   <Text>ShadowView</Text>
 * </ShadowView>
 * ```
 */
export const ShadowView = UnifiedShadowView;

/**
 * LinearShadowView: for a linear gradient background shadow
 * (requires e.g. colors, from, to).
 *
 * @remarks
 * See {@link LinearInnerShadowProps} for a solid background shadow.
 *
 * @example
 * ```ts
 *  <LinearShadowView
 *    style={styles.shadowView}
 *    colors={['#f1c40f', '#e74c3c']}
 *    from="top"
 *  >
 *    <Text>LinearShadowView</Text>
 *  </LinearShadowView>
 * ```
 */
export const LinearShadowView = UnifiedShadowView;
//# sourceMappingURL=ShadowView.js.map